
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "sprites.h"
#include "cpcrslib.h"

// DEFINE BOOL
typedef char bool;
#define true 1
#define false 0


#define _sizeMode1X 320
#define _sizeMode1Y 200

#define _tamMapX 320
#define _tamMapY 200

#define cursorUp 0
#define cursorRight 1
#define cursorDown 2
#define cursorLeft 8
#define keyZ 71
#define spaceBar 47
#define max_shots 5
#define max_guard_shots 4

// Numero de teclas
#define KEYBOARD_HW_LINES 10
//Array to handle the 80 physical keys bit state
unsigned char aKeyboard[KEYBOARD_HW_LINES];

typedef struct
{
    unsigned char actualState;
    unsigned char actualFrame;
    unsigned char numFrames; // Numero de frams que tienen todas las animaciones (3?)
    unsigned char numStates; // Numero de estados que tiene la animacion, moviendoseDerecha, moviendoseArriba...
    unsigned char tickDelay; // Ciclos en los que se va a retrasar el cambio de sprite
    unsigned int tickLimit;
    unsigned char sizeSpriteX;
    unsigned char sizeSpriteY;
    char* sprites[4][4];
} Sprite;

typedef struct {
    unsigned char lifes;
    unsigned int score;
    bool continueGame;
    bool active;
    bool completed;
    unsigned char level;
} Game;

typedef struct {
    unsigned char sizeX;
    unsigned char sizeY;
    short posX;
    short posY;
    Sprite sprite;
} Disk;

typedef struct
{
    short posX;
    short posY;
    unsigned char sizeX;
    unsigned char sizeY;
    unsigned char dir;
    unsigned char speed;
    Sprite sprite;
    bool isMoving;
    bool enabled;
	unsigned char type;
	unsigned char state;
	unsigned int nexTimeShoot;
	unsigned int shootDelay;
	unsigned char envelopes;
} Character;

typedef struct
{
    short posX;
    short posY;
    unsigned char sizeX;
    unsigned char sizeY;
    unsigned char dir;
    unsigned char speed;
    Sprite sprite;
    bool enabled;
} Shot;

//Variable auxiliar necesaria para asignar las posiciones a los guardias
unsigned char mapaPosiciones[10][20];


Character objects[10];
Character guards[15];
Shot shots[max_shots];
Shot guardShots[max_guard_shots];
int countshots;
int countGuardShots;


// Funcion para obtener el tiempo (en N de ciclos (creo))
unsigned char char3,char4;
unsigned int GetTime()
{
  unsigned int nTime = 0;

  __asm
    CALL #0xBD0D ;KL TIME PLEASE
    PUSH HL
    POP DE
    LD HL, #_char3
    LD (HL), D
    LD HL, #_char4
    LD (HL), E
  __endasm;

  nTime = (char3 << 8) + char4;

  return nTime;
}

void putSpriteMode0PRO(const char *pSprite, const short nX, const short nY, const char sizeX, const char sizeY)
{
        cpc_PutSp(pSprite,
          sizeY,
          sizeX,
          (0xC000 + ((nY / 8u) * 80u) + ((nY % 8u) * 2048u) + nX/4));
 }

void putSpriteMode0PROTRANS(const char *pSprite, const short nX, const short nY, const char sizeX, const char sizeY)
{
        cpc_PutSpTr(pSprite,
          sizeY,
          sizeX,
          (0xC000 + ((nY / 8u) * 80u) + ((nY % 8u) * 2048u) + nX/4));
 }


void SetColor(unsigned char nColorIndex, unsigned char nPaletteIndex)
{
  __asm
    ld a, 4 (ix)
    ld b, 5 (ix)
    ld c, b
    call #0xBC32 ;SCR SET INK
    __endasm;
}

/*void SetCursor(unsigned char nColum, unsigned char nLine)
{
  __asm
    ld h, 4 (ix)
    ld l, 5 (ix)
    call #0xBB75 ;TXT SET CURSOR
  __endasm;
}

void SetPalette(const unsigned char *pPalette)
{
  unsigned char nColor = 0;

  for(nColor = 0; nColor < NUM_COLORS; nColor++)
    SetColor(nColor, pPalette[nColor]);
}*/


void putSpriteMode0(unsigned char *pSprite, unsigned short nX, unsigned short nY, unsigned char sizeX, unsigned char sizeY)
{
	unsigned char nYPos = 0;
	unsigned char *pAddress = 0;


	for(nYPos = 0; nYPos < sizeY; nYPos++)
	{
		pAddress = (unsigned char *)(0xC000 + ((nY / 8u) * 80u) + ((nY % 8u) * 2048u) + nX/4);

		memcpy(pAddress, pSprite, sizeX);
		pSprite += sizeX;
		nY++;
	}
}

void drawShot() {
    unsigned char i = 0;

    for(i = 0; i < max_shots; i++) {
        if(shots[i].enabled) {
            putSpriteMode0PRO(shots[i].sprite.sprites[0][0], shots[i].posX, shots[i].posY, shots[i].sprite.sizeSpriteX, shots[i].sprite.sizeSpriteY);

            // Actualizamos el nuevo frame si es el momento
            if(GetTime() > shots[i].sprite.tickLimit)
            {
                shots[i].sprite.actualFrame++;
                if(shots[i].sprite.actualFrame >= shots[i].sprite.numFrames) shots[i].sprite.actualFrame=0;
                shots[i].sprite.tickLimit = GetTime() + shots[i].sprite.tickDelay;
            }
        }
    }

   for(i = 0; i <max_guard_shots ; i++) {
        if(guardShots[i].enabled) {
            putSpriteMode0PRO(guardShots[i].sprite.sprites[0][0], guardShots[i].posX, guardShots[i].posY, guardShots[i].sprite.sizeSpriteX, guardShots[i].sprite.sizeSpriteY);

            // Actualizamos el nuevo frame si es el momento
            if(GetTime() > guardShots[i].sprite.tickLimit)
            {
                guardShots[i].sprite.actualFrame++;
                if(guardShots[i].sprite.actualFrame >= guardShots[i].sprite.numFrames) guardShots[i].sprite.actualFrame=0;
                guardShots[i].sprite.tickLimit = GetTime() + guardShots[i].sprite.tickDelay;
            }
        }
    }

}

void drawDisk(Disk *hdd) {
    putSpriteMode0(hdd->sprite.sprites[0][0], hdd->posX, hdd->posY, hdd->sprite.sizeSpriteX, hdd->sprite.sizeSpriteY);

    // Actualizamos el nuevo frame si es el momento
    if(GetTime() > hdd->sprite.tickLimit)
    {
        hdd->sprite.actualFrame++;
        if(hdd->sprite.actualFrame >= hdd->sprite.numFrames) hdd->sprite.actualFrame=0;
        hdd->sprite.tickLimit = GetTime() + hdd->sprite.tickDelay;
    }
}

// DRAW MAP AND PLAYER
void drawCharacter(Character *player)
{
    //putSpriteMode0(player->sprite.spriteSheet + player->sprite.desplazamiento, player->posX, player->posY, player->sprite.sizeSpriteX, player->sprite.sizeSpriteY);
     putSpriteMode0PRO(player->sprite.sprites[player->sprite.actualState][player->sprite.actualFrame], player->posX, player->posY, player->sprite.sizeSpriteX, player->sprite.sizeSpriteY);

    //pAddress = (unsigned char *)(0xC000 + ((nY / 8u) * 80u) + ((nY % 8u) * 2048u) + nX/4);


    // Actualizamos el nuevo frame si es el momento
    if(GetTime() > player->sprite.tickLimit && player->isMoving)
    {
        player->sprite.actualFrame++;
        player->sprite.tickLimit = GetTime() + player->sprite.tickDelay;
        if(player->sprite.actualFrame >= player->sprite.numFrames-1) player->sprite.actualFrame=0;
    }
}

// Get char from keyboard
char nGetChar;
char getChar()
{
  __asm
    LD HL, #_nGetChar
    LD (HL), #0
    CALL #0xBB09 ;KM READ CHAR
    JP NC, _end_getchar
    LD (HL), A
    _end_getchar:
  __endasm;

  return nGetChar;
}

// Lee todas las teclas y las almacena en _aKeyboard
void readKeyboard()
{
  __asm
    di
    ld hl, #_aKeyboard
    ld bc,#0xf782
    out (c),c
    ld bc,#0xf40e
    ld e,b
    out (c),c
    ld bc,#0xf6c0
    ld d,b
    out (c),c
    ld c,#0x00
    out (c),c
    ld bc,#0xf792
    out (c),c
    ld a,#0x40
    ld c,#0x4a
    _loop:
    ld b,d
    out (c),a
    ld b,e
    ini
    inc a
    cp c
    jr c,_loop
    ld bc,#0xf782
    out (c),c
    ei
  __endasm;
}

// Obtenemos >0 si la tecla esta pulsada
unsigned char isKeyPressed(enum _eKey eKey)
{
  unsigned char nKeyLine = eKey / 8;
  unsigned char nBit = eKey % 8;

  if(((unsigned char)1 << nBit & aKeyboard[nKeyLine]) == 0)
    return 1;

  return 0;
}

bool ShotCollision(Shot *actor, Character *actor2) {
     bool resultado = true;
		if (actor->posX + actor->sizeX < actor2->posX) {
            resultado = false;
        }
        if (actor->posY + actor->sizeY < actor2->posY) {
            resultado = false;
        }
        if (actor->posX > actor2->posX + actor2->sizeX) {
            resultado = false;
        }
        if (actor->posY > actor2->posY + actor2->sizeY ) {
            resultado = false;
        }
    return resultado;
}


void oppositeDirection(Character *guard){
	if(guard->dir==0){
		guard->dir=4;
	}
	else if(guard->dir==1){
		guard->dir=5;
	}
	else if(guard->dir==2){
		guard->dir=6;
	}
	else if(guard->dir==3){
		guard->dir=7;
	}
	else if(guard->dir==4){
		guard->dir=0;
	}
	else if(guard->dir==5){
		guard->dir=1;
	}
	else if(guard->dir==6){
	guard->dir=2;
	}
	else if(guard->dir==7){
	guard->dir=3;
	}

}

void deleleteEnvelop(){
	unsigned char i=0;
	for(i=0;i<max_shots;i++){
		shots[i].enabled=false;
	}
	for(i=0;i<max_guard_shots;i++){
		guardShots[i].enabled=false;
	}

}

void pintarDatos(Game *newGame, Character *player)
{
	//TXT SET CURSOR 0,0
	__asm
		ld h, #1
		ld l, #1
		call #0xBB75
	__endasm;
	printf("P:%u V:%u N:%u S:%u",  newGame->score, newGame->lifes,newGame->level, player->envelopes);
}

void shotCollision(Character *guard, Game *g, Character *player) {
    unsigned char i = 0;

    for(i = 0; i < max_shots; i++) {
        if(shots[i].enabled) {
            if(ShotCollision(&shots[i], guard) == true) {
                //guard->isMoving = false;
                //guard->enabled = false;
                shots[i].enabled = false;

                putSpriteMode0PRO(backGround,shots[i].posX, shots[i].posY, shots[i].sprite.sizeSpriteX, shots[i].sprite.sizeSpriteY);

                g->score = g->score + 10;
				pintarDatos(g,player);
				if(guard->state!=1)
					oppositeDirection(guard);
				guard->state=1;
				 if(guard->sprite.actualState == 1)
				{
					guard->sprite.actualState = 2;
				}
				else
				{
					guard->sprite.actualState = 3;
				}
            }
        }
    }
}

//TODO
bool shotCollisionPlayer(Character *player)
{
    unsigned char i = 0;

    for(i = 0; i < max_guard_shots; i++)
    {
        if(guardShots[i].enabled)
        {
            if(ShotCollision(&guardShots[i], player) == true)
            {
                guardShots[i].enabled = false;
                return true;
            }
        }
    }
    return false;
}


bool collisionDetectorDisk(Character *actor, Disk *actor2) {
    bool resultado = true;
		if (actor->posX + actor->sizeX < actor2->posX) {
            resultado = false;
        }
        if (actor->posY + actor->sizeY < actor2->posY) {
            resultado = false;
        }
        if (actor->posX > actor2->posX + actor2->sizeX) {
            resultado = false;
        }
        if (actor->posY > actor2->posY + actor2->sizeY ) {
            resultado = false;
        }
    return resultado;
}

bool collisionDetector(Character *actor, Character *actor2) {
	 bool resultado = true;
		if (actor->posX + actor->sizeX < actor2->posX) {
            resultado = false;
        }
        if (actor->posY + actor->sizeY < actor2->posY) {
            resultado = false;
        }
        if (actor->posX > actor2->posX + actor2->sizeX) {
            resultado = false;
        }
        if (actor->posY > actor2->posY + actor2->sizeY ) {
            resultado = false;
        }

       if((actor2->posX < 0 || actor2->posX > _tamMapX - (actor2->sprite.sizeSpriteX+10)
           || actor2->posY < 10 || actor2->posY > _tamMapY  - actor2->sprite.sizeSpriteY) && actor2->state == 1)
        {
            putSpriteMode0PRO(backGround,actor2->posX, actor2->posY, actor2->sprite.sizeSpriteX, actor2->sprite.sizeSpriteY);
            actor2->enabled = false;
        }

    return resultado;
}

//Calcula la direcci?n por donde debe ir el enemigo para capturar al
//jugador
void calculateDirection(Character *player, Character *enemy)
{
    short X = player->posX - enemy->posX;

    short Y = player->posY - enemy->posY;
	if(enemy->state!=1)
    {
        if(X == 0 && Y<0) //Arriba

            enemy->dir = 0;

        else if(X == 0 && Y>0) //Abajo

            enemy->dir = 4;

        else if(X>0 && Y == 0) // Derecha

            enemy->dir = 2;

        else if(X<0 && Y == 0) //Izquierda

            enemy->dir = 6;

        else if(X>0 && Y<0) //Arriba-Derecha

            enemy->dir = 1;

        else if(X<0 && Y<0) // Arriba-Izquierda

            enemy->dir = 7;

        else if(X<0 && Y>0) //Abajo-Izquierda

            enemy->dir = 5;

        else                //Abajo-Derecha
            enemy->dir = 3;

        if(X>0)
        {
            enemy->sprite.actualState = 0;
        }
        else if(X<0)
        {
            enemy->sprite.actualState = 1;
        }
	}
}



//Calcula la direcci?n por donde debe ir el enemigo para capturar al
//jugador
void calculateDirectionPoor(Character *player, Character *enemy)
{
	if(enemy->state!=1){
    short X = player->posX - enemy->posX;

    short Y = player->posY - enemy->posY;

    if(X == 0 && Y<0) //Arriba

        enemy->dir = 0;

    else if(X == 0 && Y>0) //Abajo

        enemy->dir = 4;

    else if((X>0 && Y == 0) || (X>0 && Y<0) || (X>0 && Y>0)) // Derecha

        enemy->dir = 2;

    else if((X<0 && Y == 0) || (X<0 && Y<0) || (X<0 && Y>0))//Izquierda

        enemy->dir = 6;

    if(X>0)
    {
        enemy->sprite.actualState = 0;
    }
    else if(X<0)
    {
        enemy->sprite.actualState = 1;
    }
	}
}


void initGuardShot(unsigned char dire, short x, short y, unsigned char s) {
    if(countGuardShots < max_guard_shots) {
        bool notavailable = true;
        unsigned char i = 0;
        for(i = 0; i < max_guard_shots && notavailable; i++) {
            if(guardShots[i].enabled == false) {
                notavailable = false;
                guardShots[i].enabled = true;
                guardShots[i].dir = dire;
                guardShots[i].posX = x + 1;
                guardShots[i].posY = y + 1;
                guardShots[i].sizeX = 2;
                guardShots[i].sizeY = 4;
                guardShots[i].speed = s + 3;
                guardShots[i].sprite.sprites[0][0] = bullet;
                guardShots[i].sprite.numFrames = 1;
                guardShots[i].sprite.actualFrame = 1;
                guardShots[i].sprite.tickLimit = 0;
                guardShots[i].sprite.tickDelay = 40;
                guardShots[i].sprite.actualState = 0;
                guardShots[i].sprite.sizeSpriteX = 2;
                guardShots[i].sprite.sizeSpriteY = 4;
            }
        }
    }
}


void initShot(unsigned char dire, short x, short y, unsigned char s, Character *player, Game *g) {
    if(player->envelopes>0){
	if(countshots < max_shots) {
        bool notavailable = true;

        unsigned char i = 0;
        for(i = 0; i < max_shots && notavailable; i++) {
            if(shots[i].enabled == false) {
                notavailable = false;
                shots[i].enabled = true;
                shots[i].dir = dire;
                shots[i].posX = x + 1;
                shots[i].posY = y + 1;
                shots[i].sizeX = 4;
                shots[i].sizeY = 8;
                shots[i].speed = s + 3;
                shots[i].sprite.sprites[0][0] = envelop;
                shots[i].sprite.numFrames = 1;
                shots[i].sprite.actualFrame = 1;
                shots[i].sprite.tickLimit = 0;
                shots[i].sprite.tickDelay = 40;
                shots[i].sprite.actualState = 0;
                shots[i].sprite.sizeSpriteX = 4;
                shots[i].sprite.sizeSpriteY = 8;
				player->envelopes--;
				pintarDatos(g,player);
            }
        }
    }
}
}

void getZisPressed(Character *player, Game *g) {
    readKeyboard();

    if(isKeyPressed(keyZ))
    {
        if(GetTime() > player->nexTimeShoot)
        {
            initShot(player->dir, player->posX, player->posY, player->speed, player, g);
            player->nexTimeShoot = GetTime() + player->shootDelay;
        }
    }
}

// Actualizamos los flags segun las pulsaciones del teclado
void getKeys(Character *player)
{
    player->isMoving = false;
    readKeyboard();

    if(isKeyPressed(cursorUp))
    {
        player->isMoving = true;
        player-> dir = 0;
    }

    if(isKeyPressed(cursorRight))
    {
        player->isMoving = true;
        player-> dir = 2;
        player->sprite.actualState = 0;
    }

    if(isKeyPressed(cursorDown))
    {
        player->isMoving = true;
        player-> dir = 4;
    }

    if(isKeyPressed(cursorLeft))
    {
        player->isMoving = true;
        player-> dir = 6;
        player->sprite.actualState = 1;
    }

    if(isKeyPressed(cursorRight) && isKeyPressed(cursorUp))
    {
        player->isMoving = true;
        player-> dir = 1;
    }

    if(isKeyPressed(cursorRight) && isKeyPressed(cursorDown))
    {
        player->isMoving = true;
        player-> dir = 3;
    }

    if(isKeyPressed(cursorLeft) && isKeyPressed(cursorUp))
    {
        player->isMoving = true;
        player-> dir = 7;
    }

    if(isKeyPressed(cursorLeft) && isKeyPressed(cursorDown))
    {
        player->isMoving = true;
        player-> dir = 5;
    }
}

void moveShot() {
    unsigned char i = 0;
    for(i = 0; i < max_shots; i++)
    {
        switch(shots[i].dir)
        {
            case 0: shots[i].posY = shots[i].posY - shots[i].speed;
                    break;

            case 1: shots[i].posY = shots[i].posY - shots[i].speed;
                    shots[i].posX = shots[i].posX + shots[i].speed;
                    break;

            case 2: shots[i].posX = shots[i].posX + shots[i].speed;
                    break;

            case 3: shots[i].posY = shots[i].posY + shots[i].speed;
                    shots[i].posX = shots[i].posX + shots[i].speed;
                    break;

            case 4: shots[i].posY = shots[i].posY + shots[i].speed;
                    break;

            case 5: shots[i].posY = shots[i].posY + shots[i].speed;
                    shots[i].posX = shots[i].posX - shots[i].speed;
                    break;

            case 6: shots[i].posX = shots[i].posX - shots[i].speed;
                    break;

            case 7: shots[i].posY = shots[i].posY - shots[i].speed;
                    shots[i].posX = shots[i].posX - shots[i].speed;
                    break;
        }

        if(shots[i].posX < 0 || shots[i].posX > _tamMapX - (shots[i].sprite.sizeSpriteX+4)
           || shots[i].posY < 10 || shots[i].posY > _tamMapY  - shots[i].sprite.sizeSpriteY)
        {
            shots[i].enabled = false;
        }
    }

    for(i = 0; i < max_guard_shots; i++)
    {
        switch(guardShots[i].dir)
        {
            case 0: guardShots[i].posY = guardShots[i].posY - guardShots[i].speed;
                    break;

            case 1: guardShots[i].posY = guardShots[i].posY - guardShots[i].speed;
                    guardShots[i].posX = guardShots[i].posX + guardShots[i].speed;
                    break;

            case 2: guardShots[i].posX = guardShots[i].posX + guardShots[i].speed;
                    break;

            case 3: guardShots[i].posY = guardShots[i].posY + guardShots[i].speed;
                    guardShots[i].posX = guardShots[i].posX + guardShots[i].speed;
                    break;

            case 4: guardShots[i].posY = guardShots[i].posY + guardShots[i].speed;
                    break;

            case 5: guardShots[i].posY = guardShots[i].posY + guardShots[i].speed;
                    guardShots[i].posX = guardShots[i].posX - guardShots[i].speed;
                    break;

            case 6: guardShots[i].posX = guardShots[i].posX - guardShots[i].speed;
                    break;

            case 7: guardShots[i].posY = guardShots[i].posY - guardShots[i].speed;
                    guardShots[i].posX = guardShots[i].posX - guardShots[i].speed;
                    break;
        }

        if(guardShots[i].posX < 0 || guardShots[i].posX > _tamMapX - (guardShots[i].sprite.sizeSpriteX+4)
           || guardShots[i].posY < 10 || guardShots[i].posY > _tamMapY  - guardShots[i].sprite.sizeSpriteY)
        {
            guardShots[i].enabled = false;
        }
    }
}

void clearShots()
{
    unsigned char i = 0;
    for(i = 0; i < max_shots; i++)
    {
        if(shots[i].enabled) putSpriteMode0PRO(backGround,shots[i].posX, shots[i].posY, shots[i].sprite.sizeSpriteX, shots[i].sprite.sizeSpriteY);
    }

    for(i = 0; i < max_guard_shots; i++)
    {
        if(guardShots[i].enabled) putSpriteMode0PRO(backGround,guardShots[i].posX, guardShots[i].posY, guardShots[i].sprite.sizeSpriteX, guardShots[i].sprite.sizeSpriteY);
    }

}

// CHEK THE PRESSED KEY AND WALL COLLISIONS
void moveCharacter(Character *character)
{
    if(character->isMoving)
    {
        putSpriteMode0PRO(backGround,character->posX, character->posY, character->sprite.sizeSpriteX, character->sprite.sizeSpriteY);
        switch(character->dir)
        {
            case 0: character->posY = character->posY - character->speed;
                    break;

            case 1: character->posY = character->posY - character->speed;
                    character->posX = character->posX + character->speed;
                    break;

            case 2: character->posX = character->posX + character->speed;
                    break;

            case 3: character->posY = character->posY + character->speed;
                    character->posX = character->posX + character->speed;
                    break;

            case 4: character->posY = character->posY + character->speed;
                    break;

            case 5: character->posY = character->posY + character->speed;
                    character->posX = character->posX - character->speed;
                    break;

            case 6: character->posX = character->posX - character->speed;
                    break;

            case 7: character->posY = character->posY - character->speed;
                    character->posX = character->posX - character->speed;
                    break;
        }


        // Check map bounds
        if(character->posX < 0) character->posX = 0;
        if(character->posX > _tamMapX - character->sizeX) character->posX = _tamMapX - character->sizeX;
        if(character->posY < 7) character->posY = 7;
        if(character->posY > _tamMapY  - character->sizeY) character->posY = _tamMapY - character->sizeY;
    }
}


// CHEK THE PRESSED KEY AND WALL COLLISIONS
void moveGuard(Character *character)
{
    if(character->isMoving)
    {
        putSpriteMode0PRO(backGround,character->posX, character->posY, character->sprite.sizeSpriteX, character->sprite.sizeSpriteY);
        switch(character->dir)
        {
            case 0: character->posY = character->posY - character->speed;
                    break;

            case 1: character->posY = character->posY - character->speed;
                    character->posX = character->posX + character->speed;
                    break;

            case 2: character->posX = character->posX + character->speed;
                    break;

            case 3: character->posY = character->posY + character->speed;
                    character->posX = character->posX + character->speed;
                    break;

            case 4: character->posY = character->posY + character->speed;
                    break;

            case 5: character->posY = character->posY + character->speed;
                    character->posX = character->posX - character->speed;
                    break;

            case 6: character->posX = character->posX - character->speed;
                    break;

            case 7: character->posY = character->posY - character->speed;
                    character->posX = character->posX - character->speed;
                    break;
        }
    }

    if(character->state == 0)
    {
        if(GetTime() > character->nexTimeShoot)
        {
            if(countGuardShots < max_guard_shots)
            {
                initGuardShot(character->dir, character->posX, character->posY, character->speed);
                character->nexTimeShoot = character->shootDelay + GetTime() + randi() % character->shootDelay*4;
            }
        }
    }
}

//Metodo para iniciar la matriz todo a ceros
void iniciarPosiciones()
{
    unsigned char i;
    unsigned char j;
    for(i = 0;i<10;i++)
    {
        for(j = 0; j<20; j++)
        {
            mapaPosiciones [i][j] = 0;
        }
    }
}


static unsigned long int next = 1;

int randi(void)
{
    next = next * 1103515245UL + 12345;
    return (unsigned int)(next/65536) % (RAND_MAX + 1U);
}

void srand(unsigned int seed)
{
    next = seed;
}


// MODULO EN CUARENTENA: ADRI MANCO!
//El método se encarga de posicionar los guardias, puede ampliarse su funcionalidad para anyadir objetos cuando
//tengamos el código listo. Por ahora se le pasa un número de guardas y el array de guardias y el les da su posición
void posicionarGuardias(unsigned char numGuardias, Character *enemy)
{
    unsigned char i = 0;
    //Posición X e Y que tomara el guardia
    unsigned char aleatorioX;
    unsigned char aleatorioY;

    //Iniciamos la semilla del ramdom
    iniciarPosiciones();
	srand(GetTime());
    //Bule encargado de asignar las posiciones asociadas a los guardas
    while(i<numGuardias)
    {
        aleatorioY = 2 + randi() % 8;
        //El +2 evita que salga el guarda en la esquina izquierda del mapa, y
        //el con el 16 evitamos que salga en la esquina derecha
        aleatorioX = 7 + randi() % 11;
        if(aleatorioY > (_tamMapY - 25)) {
            aleatorioY = aleatorioY - 25;
        }

        if(mapaPosiciones[aleatorioY][aleatorioX] == 0)
        {
            mapaPosiciones[aleatorioY][aleatorioX] = 1;
            enemy[i].posX = aleatorioX * 16;
            enemy[i].posY = aleatorioY * 20;
            i++;
        }

    }
}

void posicionarObjeto(Disk *hdd)
{
	//Posición X e Y que tomara el objeto
    unsigned char aleatorioX = 17 + randi() % 2;
    unsigned char aleatorioY = 1 + randi() % 8;

	hdd->posX = aleatorioX * 16;
	hdd->posY = aleatorioY * 20;
}

void init(Character *player, Character guard[], Disk *hdd)
{
     unsigned char i = 0;
    unsigned char j = 0;
    player->sprite.numFrames = 4;
    player->sprite.numStates = 2;
    player->sprite.actualFrame = 1;
    player->sprite.actualState= 0;
    player->sprite.tickLimit = 0;
    player->sprite.tickDelay = 20;
    player->sprite.sizeSpriteX = 5;
    player->sprite.sizeSpriteY = 20;
	player->state = 0;
	player->type = 'p';

    player->posX = 20;
    player->posY = 20;
    player->sizeY = 20;
    player->sizeX = 20;
    player->speed = 3;
    player->nexTimeShoot = 0;
    player->shootDelay = 100;


    player->sprite.sprites[0][0] = (char *)playerSpriteSheet;
    player->sprite.sprites[0][1] = (char *)playerSpriteSheet + player->sprite.sizeSpriteX * player->sprite.sizeSpriteY;
    player->sprite.sprites[0][2] = (char *)playerSpriteSheet + player->sprite.sizeSpriteX * player->sprite.sizeSpriteY * 2;
    player->sprite.sprites[0][3] = (char *)playerSpriteSheet + player->sprite.sizeSpriteX * player->sprite.sizeSpriteY * 3;
    player->sprite.sprites[1][0] = (char *)playerSpriteSheet + player->sprite.sizeSpriteX * player->sprite.sizeSpriteY * 4;
    player->sprite.sprites[1][1] = (char *)playerSpriteSheet + player->sprite.sizeSpriteX * player->sprite.sizeSpriteY * 5;
    player->sprite.sprites[1][2] = (char *)playerSpriteSheet + player->sprite.sizeSpriteX * player->sprite.sizeSpriteY * 6;
    player->sprite.sprites[1][3] = (char *)playerSpriteSheet + player->sprite.sizeSpriteX * player->sprite.sizeSpriteY * 7;
    player->sprite.sprites[2][0] = (char *)playerSpriteSheet;
    player->sprite.sprites[3][0] = (char *)playerSpriteSheet;

    hdd->sizeX = 10;
    hdd->sizeY = 20;
    hdd->sprite.sprites[0][0] = diskSprite;
    hdd->sprite.numFrames = 1;
    hdd->sprite.actualFrame = 1;
    hdd->sprite.tickLimit = 0;
    hdd->sprite.tickDelay = 100;
    hdd->sprite.actualState = 0;
    hdd->sprite.sizeSpriteX = 5;
    hdd->sprite.sizeSpriteY = 20;

for(i = 0; i<15;i++)
	{
//	guard[i].sprite.spriteSheet = playerSpriteSheet;
        guard[i].sprite.numFrames = 3;
        guard[i].sprite.actualFrame = 1;
        guard[i].sprite.actualState= 0;
        guard[i].sprite.tickLimit = 0;
        guard[i].sprite.tickDelay = 40;
        guard[i].sprite.sizeSpriteX = 5;
        guard[i].sprite.sizeSpriteY = 20;
        guard[i].sizeY = 20;
        guard[i].sizeX = 20;
        guard[i].speed = 1;
        guard[i].isMoving = true;
        guard[i].enabled = true;
		guard[i].state = 0;
		guard[i].type = 'g';

        guard[i].shootDelay = 500;
        guard[i].nexTimeShoot = GetTime() + randi()% guard[i].shootDelay *3;

        guard[i].sprite.sprites[0][0] = (char *)guardiaSpriteSheet;
        guard[i].sprite.sprites[0][1] = (char *)guardiaSpriteSheet + guard[i].sprite.sizeSpriteX * guard[i].sprite.sizeSpriteY;
        guard[i].sprite.sprites[0][2] = (char *)guardiaSpriteSheet + guard[i].sprite.sizeSpriteX * guard[i].sprite.sizeSpriteY * 2;
        guard[i].sprite.sprites[0][3] = (char *)guardiaSpriteSheet + guard[i].sprite.sizeSpriteX * guard[i].sprite.sizeSpriteY * 3;
        guard[i].sprite.sprites[1][0] = (char *)guardiaSpriteSheet + guard[i].sprite.sizeSpriteX * guard[i].sprite.sizeSpriteY * 4;
        guard[i].sprite.sprites[1][1] = (char *)guardiaSpriteSheet + guard[i].sprite.sizeSpriteX * guard[i].sprite.sizeSpriteY * 5;
        guard[i].sprite.sprites[1][2] = (char *)guardiaSpriteSheet + guard[i].sprite.sizeSpriteX * guard[i].sprite.sizeSpriteY * 6;
        guard[i].sprite.sprites[1][3] = (char *)guardiaSpriteSheet + guard[i].sprite.sizeSpriteX * guard[i].sprite.sizeSpriteY * 7;
        guard[i].sprite.sprites[2][0] = (char *)guardiaSpriteSheet + guard[i].sprite.sizeSpriteX * guard[i].sprite.sizeSpriteY * 8;
		guard[i].sprite.sprites[2][1] = (char *)guardiaSpriteSheet + guard[i].sprite.sizeSpriteX * guard[i].sprite.sizeSpriteY * 9;
		guard[i].sprite.sprites[2][2] = (char *)guardiaSpriteSheet + guard[i].sprite.sizeSpriteX * guard[i].sprite.sizeSpriteY * 10;
		guard[i].sprite.sprites[2][3] = (char *)guardiaSpriteSheet + guard[i].sprite.sizeSpriteX * guard[i].sprite.sizeSpriteY * 11;
		guard[i].sprite.sprites[3][0] = (char *)guardiaSpriteSheet + guard[i].sprite.sizeSpriteX * guard[i].sprite.sizeSpriteY * 12;
		guard[i].sprite.sprites[3][1] = (char *)guardiaSpriteSheet + guard[i].sprite.sizeSpriteX * guard[i].sprite.sizeSpriteY * 13;
		guard[i].sprite.sprites[3][2] = (char *)guardiaSpriteSheet + guard[i].sprite.sizeSpriteX * guard[i].sprite.sizeSpriteY * 14;
		guard[i].sprite.sprites[3][3] = (char *)guardiaSpriteSheet + guard[i].sprite.sizeSpriteX * guard[i].sprite.sizeSpriteY * 15;
	}
	posicionarGuardias(i, guard);
	posicionarObjeto(hdd);


	for(i=0; i<15; i++)
	{
	    for(j=0; j<10; j++)
        {
            putSpriteMode0PRO(backGround, i*20, j*20, 10, 20);
        }
	}
}

void effectBlackground()
{
	unsigned char i;
	unsigned char j;
	for(i=0; i<15; i++)
	{
	    for(j=0; j<10; j++)
        {
            putSpriteMode0PRO(backGround, i*20, j*20, 10, 20);
        }
	}
}

void initGame(Game *g) {
    g->active = true;
    g->continueGame = true;
    g->completed = false;
    g->score = 0;
    g->lifes = 3;
    g->level = 1;
}


int main()
{
    Character player;
    unsigned char i = 0;
    Game newGame;
    Disk hardDisk;
    unsigned char numGuards = 2;
	unsigned char sinpulsar = 0;
	char passedTime = 0;
	char initTime = 0;
	//Variable que se suma a la velocidad inicial de los guardias a partir de nivel 5
	unsigned char inc_vel = 0;
	unsigned char END = false;
    //antes de aqui poner las variables
    countshots = 0;


    //SCR_SET_MODE 0
    __asm
            ld a, #0
            call #0xBC0E
    __endasm;

        //PALETE
  __asm
    ld a, #0
    ld b, #1 ;black
    ld c, b
    call #0xBC32 ;SCR SET INK


    ld a, #1
    ld b, #24 ;Yellow
    ld c, b
    call #0xBC32 ;SCR SET INK

    ld a, #2
    ld b, #20 ;Pastel Yellow
    ld c, b
    call #0xBC32 ;SCR SET INK

    ld a, #3
    ld b, #6 ;Bright Yellow
    ld c, b
    call #0xBC32 ;SCR SET INK

	ld a, #4
    ld b, #26 ;Bright Yellow
    ld c, b
    call #0xBC32 ;SCR SET INK

	ld a, #5
    ld b, #0 ;Bright Yellow
    ld c, b
    call #0xBC32 ;SCR SET INK

	ld a, #6
    ld b, #2 ;Bright Yellow
    ld c, b
    call #0xBC32 ;SCR SET INK

	ld a, #7
    ld b, #1;Bright Yellow
    ld c, b
    call #0xBC32 ;SCR SET INK

	ld a, #8
    ld b, #10 ;Bright Yellow
    ld c, b
    call #0xBC32 ;SCR SET INK

	ld a, #9
    ld b, #12 ;Bright Yellow
    ld c, b
    call #0xBC32 ;SCR SET INK

	ld a, #10
    ld b, #14 ;Bright Yellow
    ld c, b
    call #0xBC32 ;SCR SET INK

	ld a, #11
    ld b, #16 ;Bright Yellow
    ld c, b
    call #0xBC32 ;SCR SET INK

	ld a, #12
    ld b, #18 ;Bright Yellow
    ld c, b
    call #0xBC32 ;SCR SET INK

	ld a, #13
    ld b, #22 ;Bright Yellow
    ld c, b
    call #0xBC32 ;SCR SET INK

	ld a, #14
    ld b, #13 ;Bright Yellow
    ld c, b
    call #0xBC32 ;SCR SET INK

	ld a, #15
    ld b, #11 ;Bright Yellow
    ld c, b
    call #0xBC32 ;SCR SET INK

  __endasm;



  //SCR SET BORDER 0
  __asm
    ld b, #0 ;black
    ld c, b
    call #0xBC38
  __endasm;


	while(END == false)
	{

    initGame(&newGame);
	pintarDatos(&newGame,&player);

	sinpulsar=0;
    memcpy(0xC000, title, 0x3FFF);
	nGetChar = 0;
	while(getChar() != 'p'){}

    while(newGame.active) {
        newGame.continueGame = true;
        newGame.completed = false;
        init(&player, guards, &hardDisk);
        drawDisk(&hardDisk);
		
        moveShot();
		if(numGuards < 5)
		{
			numGuards=newGame.level;
		}
		else if(newGame.level>=5)
		{
			i=0;
			while(i<numGuards)
			{
			    guards[i].speed= 1 + inc_vel;
				i++;
			}
			i=0;
			if(newGame.level % 5 == 0)
			{
				inc_vel++;
			}
		}
		player.envelopes = numGuards + 1;
		pintarDatos(&newGame, &player);

        while(newGame.continueGame)
        {
			passedTime = 0;
            getKeys(&player);
            moveCharacter(&player);
            drawCharacter(&player);
            getZisPressed(&player, &newGame);
            clearShots();
            moveShot();
            drawShot();
			drawDisk(&hardDisk);


            while(i<numGuards) {
			initTime = GetTime();
                if(guards[i].enabled) {
					if(guards[i].state != 1)
					{
						shotCollision(&guards[i], &newGame, &player);
					}
                    if(i == 2 || i == 4)
                    calculateDirectionPoor(&player, &guards[i]);
                    else
                    calculateDirection(&player, &guards[i]);

                    moveGuard(&guards[i]);
                    drawCharacter(&guards[i]);

                    // Restamos una vida si el jugador colisiona con un guardia, o con una bala
                    if(collisionDetector(&player, &guards[i]) == true && guards[i].state != 1
                        || (i==0 && shotCollisionPlayer(&player)))
                    {
                            newGame.continueGame = false;
                            if(newGame.lifes == 0)
                            {
                                newGame.active = false;
                            }
                            newGame.lifes--;
							deleleteEnvelop();
							//Pintar score
							pintarDatos(&newGame, &player);
							//Fin de pintado
                    }
                }
                i++;
				while(passedTime < 8){
			passedTime = GetTime() - initTime;
			}
           }

           if(collisionDetectorDisk(&player, &hardDisk) == true) {
               newGame.continueGame = false;
               newGame.completed = true;
			   deleleteEnvelop();
           }

           i = 0;
        }

        if(newGame.completed) {
            newGame.score = newGame.score + (100 * newGame.level);
            newGame.level++;
        }
        //aqui deberia mostrar las puntuacion final si es el final del juego o la puntuacion obtenida y total antes de pasar de nivel
        // de todas formas limpiar el mapa antes
    }
	
	}
    return 0;
}
